from RPi import GPIO  # type: ignore
import time

from bluer_options import string
from bluer_sbc.session.functions import reply_to_bash

from bluer_ugv.swallow.session.classical.leds import ClassicalLeds
from bluer_ugv.logger import logger

BUTTON_PRESS_DURATION_UPDATE = 5
BUTTON_PRESS_DURATION_SHUTDOWN = 10
BUTTON_PRESS_DURATION_IGNORE = 15


class ClassicalPushButton:
    def __init__(self, leds: ClassicalLeds):
        self.pin = 26
        self.state = False

        logger.info(
            "{}: GPIO#{}.".format(
                self.__class__.__name__,
                self.pin,
            )
        )

        self.press_time: int = 0
        self.press_duration: int = -1

        self.leds = leds

    def initialize(self) -> bool:
        try:
            GPIO.setup(
                self.pin,
                GPIO.IN,
                pull_up_down=GPIO.PUD_UP,
            )
        except Exception as e:
            logger.error(e)
            return False

        return True

    def update(self) -> bool:
        button_pressed = not GPIO.input(self.pin)
        if button_pressed:
            if not self.state:
                logger.info("push button pressed.")
                self.press_time = time.time()

            self.leds.leds["yellow"]["state"] = True

            self.press_duration = time.time() - self.press_time
            if self.press_duration > BUTTON_PRESS_DURATION_IGNORE:
                self.leds.leds["red"]["state"] = False
            elif self.press_duration > BUTTON_PRESS_DURATION_SHUTDOWN:
                self.leds.leds["red"]["state"] = True
            elif self.press_duration > BUTTON_PRESS_DURATION_UPDATE:
                self.leds.leds["red"]["state"] = not self.leds.leds["red"]["state"]
        elif self.state:
            logger.info(
                "push button released after {}.".format(
                    string.pretty_duration(
                        self.press_duration,
                    )
                )
            )

            self.leds.leds["yellow"]["state"] = False

            if self.press_duration < BUTTON_PRESS_DURATION_IGNORE:
                if self.press_duration > BUTTON_PRESS_DURATION_SHUTDOWN:
                    reply_to_bash("shutdown")
                    return False

                if self.press_duration > BUTTON_PRESS_DURATION_UPDATE:
                    reply_to_bash("update")
                    return False

        self.state = button_pressed

        return True



from RPi import GPIO  # type: ignore

from bluer_ugv.swallow.session.classical.leds import ClassicalLeds
from bluer_ugv.swallow.session.classical.setpoint import ClassicalSetPoint
from bluer_ugv.logger import logger


class GenericMotor:
    def __init__(
        self,
        role: str,
        lpwm_pin: int,
        rpwm_pin: int,
        setpoint: ClassicalSetPoint,
        leds: ClassicalLeds,
    ):
        self.role = role
        self.lpwm_pin = lpwm_pin
        self.rpwm_pin = rpwm_pin
        self.setpoint = setpoint
        self.leds = leds
        self.state: int = 0

        self.lpwm = None
        self.rpwm = None

        # Internal state for jitter protection
        self._last_direction = 0  # -1 for reverse, 0 for stop, 1 for forward
        self._last_duty = 0  # Last applied duty cycle

        logger.info(f"{self.__class__.__name__}: {role}")

    def cleanup(self):
        try:
            if self.lpwm:
                self.lpwm.stop()
                self.lpwm = None
            if self.rpwm:
                self.rpwm.stop()
                self.rpwm = None
        except Exception as e:
            logger.warning(e)

        logger.info(f"{self.__class__.__name__}.cleanup")

    def initialize(self) -> bool:
        GPIO.setup(self.lpwm_pin, GPIO.OUT)
        GPIO.setup(self.rpwm_pin, GPIO.OUT)

        self.lpwm = GPIO.PWM(self.lpwm_pin, 1000)  # 1 kHz
        self.rpwm = GPIO.PWM(self.rpwm_pin, 1000)

        self.lpwm.start(0)
        self.rpwm.start(0)

        logger.info(
            "{}: {} motor initialized on LPWM=GPIO#{}, RPWM=GPIO#{}".format(
                self.__class__.__name__,
                self.role,
                self.lpwm_pin,
                self.rpwm_pin,
            )
        )
        return True

    def update(self) -> bool:
        self.state = self.setpoint.get(what=self.role)

        value = max(-100, min(100, self.state))  # Clamp
        duty_cycle = abs(value)

        direction = 0
        if value > 0:
            direction = 1
        elif value < 0:
            direction = -1

        if direction == self._last_direction and duty_cycle == self._last_duty:
            return True

        if direction == 1:
            self.rpwm.ChangeDutyCycle(duty_cycle)
            self.lpwm.ChangeDutyCycle(0)
        elif direction == -1:
            self.rpwm.ChangeDutyCycle(0)
            self.lpwm.ChangeDutyCycle(duty_cycle)
        else:
            self.rpwm.ChangeDutyCycle(0)
            self.lpwm.ChangeDutyCycle(0)

        self._last_direction = direction
        self._last_duty = duty_cycle

        logger.info(
            "{}.update: {} {}".format(
                self.__class__.__name__,
                "->" if direction == 1 else "<-",
                duty_cycle,
            )
        )

        self.leds.leds["red"]["state"] = not self.leds.leds["red"]["state"]

        return True
